@@ -2,7 +2,7 @@
  * Programmer(s): Daniel R. Reynolds @ SMU
  *---------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2021, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -40,9 +40,10 @@
     SDIRK -- singly-diagonally implicit Runge Kutta
     ESDIRK -- explicit [1st stage] singly-diagonally implicit Runge Kutta
   The 'A-stable' and 'L-stable' columns are based on numerical estimates
-  of each property.  The 'QP' column denotes whether the coefficients
-  of the method are known precisely enough for use in 'long double'
-  (128-bit) calculations.
+  of each property.
+
+  The 'QP' column denotes whether the coefficients of the method are known
+  precisely enough for use in quad precision (128-bit) calculations.
 
      imeth                       type  A-stable  L-stable  QP
     ----------------------------------------------------------
@@ -56,8 +57,10 @@
      SDIRK_5_3_4                SDIRK     Y         Y       Y
      KVAERNO_5_3_4             ESDIRK     Y         N       N
      ARK436L2SA_DIRK_6_3_4*    ESDIRK     Y         Y       N
+     ARK437L2SA_DIRK_7_3_4*    ESDIRK     Y         Y       N
      KVAERNO_7_4_5             ESDIRK     Y         Y       N
      ARK548L2SA_DIRK_8_4_5*    ESDIRK     Y         Y       N
+     ARK548L2SAb_DIRK_8_4_5*   ESDIRK     Y         Y       N
     ----------------------------------------------------------
 
   ---------------------------------------------------------------*/
@@ -383,6 +386,57 @@ ARKodeButcherTable ARKodeButcherTable_LoadDIRK(int imethod)
     B->d[5] = RCONST(61727.0)/RCONST(225920.0);
     break;
 
+  case(ARK437L2SA_DIRK_7_3_4):    /* ARK4(3)7L[2]SA-ESDIRK */
+    B = ARKodeButcherTable_Alloc(7, SUNTRUE);
+    B->q = 4;
+    B->p = 3;
+
+    B->A[1][0] = RCONST(1235.0)/RCONST(10000.0);
+    B->A[1][1] = RCONST(1235.0)/RCONST(10000.0);
+    B->A[2][0] = RCONST(624185399699.0)/RCONST(4186980696204.0);
+    B->A[2][1] = RCONST(624185399699.0)/RCONST(4186980696204.0);
+    B->A[2][2] = RCONST(1235.0)/RCONST(10000.0);
+    B->A[3][0] = RCONST(1258591069120.0)/RCONST(10082082980243.0);
+    B->A[3][1] = RCONST(1258591069120.0)/RCONST(10082082980243.0);
+    B->A[3][2] = RCONST(-322722984531.0)/RCONST(8455138723562.0);
+    B->A[3][3] = RCONST(1235.0)/RCONST(10000.0);
+    B->A[4][0] = RCONST(-436103496990.0)/RCONST(5971407786587.0);
+    B->A[4][1] = RCONST(-436103496990.0)/RCONST(5971407786587.0);
+    B->A[4][2] = RCONST(-2689175662187.0)/RCONST(11046760208243.0);
+    B->A[4][3] = RCONST(4431412449334.0)/RCONST(12995360898505.0);
+    B->A[4][4] = RCONST(1235.0)/RCONST(10000.0);
+    B->A[5][0] = RCONST(-2207373168298.0)/RCONST(14430576638973.0);
+    B->A[5][1] = RCONST(-2207373168298.0)/RCONST(14430576638973.0);
+    B->A[5][2] = RCONST(242511121179.0)/RCONST(3358618340039.0);
+    B->A[5][3] = RCONST(3145666661981.0)/RCONST(7780404714551.0);
+    B->A[5][4] = RCONST(5882073923981.0)/RCONST(14490790706663.0);
+    B->A[5][5] = RCONST(1235.0)/RCONST(10000.0);
+    B->A[6][2] = RCONST(9164257142617.0)/RCONST(17756377923965.0);
+    B->A[6][3] = RCONST(-10812980402763.0)/RCONST(74029279521829.0);
+    B->A[6][4] = RCONST(1335994250573.0)/RCONST(5691609445217.0);
+    B->A[6][5] = RCONST(2273837961795.0)/RCONST(8368240463276.0);
+    B->A[6][6] = RCONST(1235.0)/RCONST(10000.0);
+
+    B->b[2] = RCONST(9164257142617.0)/RCONST(17756377923965.0);
+    B->b[3] = RCONST(-10812980402763.0)/RCONST(74029279521829.0);
+    B->b[4] = RCONST(1335994250573.0)/RCONST(5691609445217.0);
+    B->b[5] = RCONST(2273837961795.0)/RCONST(8368240463276.0);
+    B->b[6] = RCONST(1235.0)/RCONST(10000.0);
+
+    B->c[1] = RCONST(247.0)/RCONST(1000.0);
+    B->c[2] = RCONST(4276536705230.0)/RCONST(10142255878289.0);
+    B->c[3] = RCONST(67.0)/RCONST(200.0);
+    B->c[4] = RCONST(3.0)/RCONST(40.0);
+    B->c[5] = RCONST(7.0)/RCONST(10.0);
+    B->c[6] = RCONST(1.0);
+
+    B->d[2] = RCONST(4469248916618.0)/RCONST(8635866897933.0);
+    B->d[3] = RCONST(-621260224600.0)/RCONST(4094290005349.0);
+    B->d[4] = RCONST(696572312987.0)/RCONST(2942599194819.0);
+    B->d[5] = RCONST(1532940081127.0)/RCONST(5565293938103.0);
+    B->d[6] = RCONST(2441.0)/RCONST(20000.0);
+    break;
+
   case(KVAERNO_7_4_5):    /* Kvaerno(7,4,5)-ESDIRK */
     B = ARKodeButcherTable_Alloc(7, SUNTRUE);
     B->q = 5;
@@ -491,6 +545,67 @@ ARKodeButcherTable ARKodeButcherTable_LoadDIRK(int imethod)
     B->c[7] = RCONST(1.0);
     break;
 
+  case(ARK548L2SAb_DIRK_8_4_5):    /* ARK5(4)8L[2]SAb-ESDIRK */
+    B = ARKodeButcherTable_Alloc(8, SUNTRUE);
+    B->q = 5;
+    B->p = 4;
+    B->A[1][0] = RCONST(2.0)/RCONST(9.0);
+    B->A[1][1] = RCONST(2.0)/RCONST(9.0);
+    B->A[2][0] = RCONST(2366667076620.0)/RCONST(8822750406821.0);
+    B->A[2][1] = RCONST(2366667076620.0)/RCONST(8822750406821.0);
+    B->A[2][2] = RCONST(2.0)/RCONST(9.0);
+    B->A[3][0] = RCONST(-257962897183.0)/RCONST(4451812247028.0);
+    B->A[3][1] = RCONST(-257962897183.0)/RCONST(4451812247028.0);
+    B->A[3][2] = RCONST(128530224461.0)/RCONST(14379561246022.0);
+    B->A[3][3] = RCONST(2.0)/RCONST(9.0);
+    B->A[4][0] = RCONST(-486229321650.0)/RCONST(11227943450093.0);
+    B->A[4][1] = RCONST(-486229321650.0)/RCONST(11227943450093.0);
+    B->A[4][2] = RCONST(-225633144460.0)/RCONST(6633558740617.0);
+    B->A[4][3] = RCONST(1741320951451.0)/RCONST(6824444397158.0);
+    B->A[4][4] = RCONST(2.0)/RCONST(9.0);
+    B->A[5][0] = RCONST(621307788657.0)/RCONST(4714163060173.0);
+    B->A[5][1] = RCONST(621307788657.0)/RCONST(4714163060173.0);
+    B->A[5][2] = RCONST(-125196015625.0)/RCONST(3866852212004.0);
+    B->A[5][3] = RCONST(940440206406.0)/RCONST(7593089888465.0);
+    B->A[5][4] = RCONST(961109811699.0)/RCONST(6734810228204.0);
+    B->A[5][5] = RCONST(2.0)/RCONST(9.0);
+    B->A[6][0] = RCONST(2036305566805.0)/RCONST(6583108094622.0);
+    B->A[6][1] = RCONST(2036305566805.0)/RCONST(6583108094622.0);
+    B->A[6][2] = RCONST(-3039402635899.0)/RCONST(4450598839912.0);
+    B->A[6][3] = RCONST(-1829510709469.0)/RCONST(31102090912115.0);
+    B->A[6][4] = RCONST(-286320471013.0)/RCONST(6931253422520.0);
+    B->A[6][5] = RCONST(8651533662697.0)/RCONST(9642993110008.0);
+    B->A[6][6] = RCONST(2.0)/RCONST(9.0);
+    B->A[7][2] = RCONST(3517720773327.0)/RCONST(20256071687669.0);
+    B->A[7][3] = RCONST(4569610470461.0)/RCONST(17934693873752.0);
+    B->A[7][4] = RCONST(2819471173109.0)/RCONST(11655438449929.0);
+    B->A[7][5] = RCONST(3296210113763.0)/RCONST(10722700128969.0);
+    B->A[7][6] = RCONST(-1142099968913.0)/RCONST(5710983926999.0);
+    B->A[7][7] = RCONST(2.0)/RCONST(9.0);
+
+    B->b[2] = RCONST(3517720773327.0)/RCONST(20256071687669.0);
+    B->b[3] = RCONST(4569610470461.0)/RCONST(17934693873752.0);
+    B->b[4] = RCONST(2819471173109.0)/RCONST(11655438449929.0);
+    B->b[5] = RCONST(3296210113763.0)/RCONST(10722700128969.0);
+    B->b[6] = RCONST(-1142099968913.0)/RCONST(5710983926999.0);
+    B->b[7] = RCONST(2.0)/RCONST(9.0);
+
+    B->d[2] = RCONST(520639020421.0)/RCONST(8300446712847.0);
+    B->d[3] = RCONST(4550235134915.0)/RCONST(17827758688493.0);
+    B->d[4] = RCONST(1482366381361.0)/RCONST(6201654941325.0);
+    B->d[5] = RCONST(5551607622171.0)/RCONST(13911031047899.0);
+    B->d[6] = RCONST(-5266607656330.0)/RCONST(36788968843917.0);
+    B->d[7] = RCONST(1074053359553.0)/RCONST(5740751784926.0);
+
+    B->c[1] = RCONST(4.0)/RCONST(9.0);
+    B->c[2] = RCONST(6456083330201.0)/RCONST(8509243623797.0);
+    B->c[3] = RCONST(1632083962415.0)/RCONST(14158861528103.0);
+    B->c[4] = RCONST(6365430648612.0)/RCONST(17842476412687.0);
+    B->c[5] = RCONST(18.0)/RCONST(25.0);
+    B->c[6] = RCONST(191.0)/RCONST(200.0);
+    B->c[7] = RCONST(1.0);
+    break;
+
   default:
 
     arkProcessError(NULL, ARK_ILL_INPUT, "ARKode",
