@@ -1,38 +1,40 @@
-/*
+/* -----------------------------------------------------------------
+ * Programmer(s): Scott D. Cohen, Alan C. Hindmarsh, Radu Serban and
+ *                Dan Shumaker @ LLNL
+ *                Daniel Reynolds @ SMU
  * -----------------------------------------------------------------
- * $Revision: 1.13 $
- * $Date: 2007/11/26 16:19:59 $
- * -----------------------------------------------------------------
- * Programmer(s): Scott D. Cohen, Alan C. Hindmarsh, Radu Serban
- *                and Dan Shumaker @ LLNL
- * -----------------------------------------------------------------
- * Copyright (c) 2002, The Regents of the University of California.
- * Produced at the Lawrence Livermore National Laboratory.
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * and Southern Methodist University.
  * All rights reserved.
- * For details, see the LICENSE file.
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
  * -----------------------------------------------------------------
  * Implementation header file for the main CVODE integrator.
- * -----------------------------------------------------------------
- */
+ * -----------------------------------------------------------------*/
 
 #ifndef _CVODE_IMPL_H
 #define _CVODE_IMPL_H
 
+#include <stdarg.h>
+
+#include "cvode/cvode.h"
+#include "cvode_proj_impl.h"
+
 #ifdef __cplusplus  /* wrapper to enable C++ usage */
 extern "C" {
 #endif
 
-#include <stdarg.h>
-
-#include <cvode/cvode.h>
-
 /*
  * =================================================================
- *   M A I N    I N T E G R A T O R    M E M O R Y    B L O C K
+ *   I N T E R N A L   C O N S T A N T S
  * =================================================================
  */
 
-/* Basic CVODE constants */
+/* Basic constants */
 
 #define ADAMS_Q_MAX 12     /* max value of q for lmm == ADAMS     */
 #define BDF_Q_MAX    5     /* max value of q for lmm == BDF       */
@@ -45,9 +47,71 @@ extern "C" {
 #define MXHNIL_DEFAULT   10             /* mxhnil default value   */
 #define MXSTEP_DEFAULT   500            /* mxstep default value   */
 
+#define MSBP 20  /* max no. of steps between lsetup calls */
+
+
+/* Control constants for lower-level functions used by cvStep
+ * ----------------------------------------------------------
+ *
+ * cvHin return values:
+ *    CV_SUCCESS
+ *    CV_RHSFUNC_FAIL
+ *    CV_TOO_CLOSE
+ *
+ * cvStep control constants:
+ *    DO_ERROR_TEST
+ *    PREDICT_AGAIN
+ *
+ * cvStep return values:
+ *    CV_SUCCESS,
+ *    CV_LSETUP_FAIL,  CV_LSOLVE_FAIL,
+ *    CV_RHSFUNC_FAIL, CV_RTFUNC_FAIL
+ *    CV_CONV_FAILURE, CV_ERR_FAILURE,
+ *    CV_FIRST_RHSFUNC_ERR
+ *
+ * cvNls input nflag values:
+ *    FIRST_CALL
+ *    PREV_CONV_FAIL
+ *    PREV_PROJ_FAIL
+ *    PREV_ERR_FAIL
+ *
+ * cvNls return values:
+ *    CV_SUCCESS,
+ *    CV_LSETUP_FAIL, CV_LSOLVE_FAIL, CV_RHSFUNC_FAIL,
+ *    CONV_FAIL, RHSFUNC_RECVR
+ *
+ * cvNewtonIteration return values:
+ *    CV_SUCCESS,
+ *    CV_LSOLVE_FAIL, CV_RHSFUNC_FAIL
+ *    CONV_FAIL, RHSFUNC_RECVR,
+ *    TRY_AGAIN
+ *
+ */
+
+#define DO_ERROR_TEST    +2
+#define PREDICT_AGAIN    +3
+
+#define TRY_AGAIN        +5
+#define FIRST_CALL       +6
+#define PREV_CONV_FAIL   +7
+#define PREV_PROJ_FAIL   +8
+#define PREV_ERR_FAIL    +9
+
+#define RHSFUNC_RECVR    +10
+#define CONSTR_RECVR     +11
+#define CONSTRFUNC_RECVR +12
+#define PROJFUNC_RECVR   +13
+
+/*
+ * =================================================================
+ *   M A I N    I N T E G R A T O R    M E M O R Y    B L O C K
+ * =================================================================
+ */
+
+
 /*
  * -----------------------------------------------------------------
- * Types : struct CVodeMemRec, CVodeMem
+ * Types: struct CVodeMemRec, CVodeMem
  * -----------------------------------------------------------------
  * The type CVodeMem is type pointer to struct CVodeMemRec.
  * This structure contains fields to keep track of problem state.
@@ -58,45 +122,53 @@ typedef struct CVodeMemRec {
 
   realtype cv_uround;    /* machine unit roundoff */
 
-  /*-------------------------- 
-    Problem Specification Data 
+  /*--------------------------
+    Problem Specification Data
     --------------------------*/
 
   CVRhsFn cv_f;              /* y' = f(t,y(t))                                */
   void *cv_user_data;        /* user pointer passed to f                      */
   int cv_lmm;                /* lmm = CV_ADAMS or CV_BDF                      */
-  int cv_iter;               /* iter = CV_FUNCTIONAL or CV_NEWTON             */
   int cv_itol;               /* itol = CV_SS, CV_SV, CV_WF, CV_NN             */
 
   realtype cv_reltol;        /* relative tolerance                            */
   realtype cv_Sabstol;       /* scalar absolute tolerance                     */
   N_Vector cv_Vabstol;       /* vector absolute tolerance                     */
-  booleantype cv_user_efun;  /* TRUE if user sets efun                        */
+  booleantype cv_atolmin0;   /* flag indicating that min(abstol) = 0          */
+  booleantype cv_user_efun;  /* SUNTRUE if user sets efun                     */
   CVEwtFn cv_efun;           /* function to set ewt                           */
   void *cv_e_data;           /* user pointer passed to efun                   */
 
+  booleantype cv_constraintsSet; /* constraints vector present:
+                                    do constraints calc                       */
+
   /*-----------------------
-    Nordsieck History Array 
+    Nordsieck History Array
     -----------------------*/
 
   N_Vector cv_zn[L_MAX];  /* Nordsieck array, of size N x (q+1).
-                             zn[j] is a vector of length N (j=0,...,q) 
-                             zn[j] = [1/factorial(j)] * h^j * (jth      
-                             derivative of the interpolating polynomial       */
+                             zn[j] is a vector of length N (j=0,...,q)
+                             zn[j] = [1/factorial(j)] * h^j *
+                             (jth derivative of the interpolating polynomial) */
 
-  /*--------------------------
-    other vectors of length N 
-    -------------------------*/
+  /*-------------------
+    Vectors of length N
+    -------------------*/
 
   N_Vector cv_ewt;     /* error weight vector                                 */
   N_Vector cv_y;       /* y is used as temporary storage by the solver
                           The memory is provided by the user to CVode
                           where the vector is named yout.                     */
   N_Vector cv_acor;    /* In the context of the solution of the nonlinear
-                          equation, acor = y_n(m) - y_n(0). On return, 
-                          this vector is scaled to give the est. local err.   */
+                          equation, acor = y_n(m) - y_n(0). On return, this
+                          vector is scaled to give the estimated local error  */
   N_Vector cv_tempv;   /* temporary storage vector                            */
   N_Vector cv_ftemp;   /* temporary storage vector                            */
+  N_Vector cv_vtemp1;  /* temporary storage vector                            */
+  N_Vector cv_vtemp2;  /* temporary storage vector                            */
+  N_Vector cv_vtemp3;  /* temporary storage vector                            */
+
+  N_Vector cv_constraints; /* vector of inequality constraint options         */
 
   /*-----------------
     Tstop information
@@ -106,12 +178,12 @@ typedef struct CVodeMemRec {
   realtype cv_tstop;
 
   /*---------
-    Step Data 
-    ---------*/  
+    Step Data
+    ---------*/
 
   int cv_q;                    /* current order                               */
-  int cv_qprime;               /* order to be used on the next step  
-                                  = q-1, q, or q+1                            */
+  int cv_qprime;               /* order to be used on the next step
+                                  qprime = q-1, q, or q+1                     */
   int cv_next_q;               /* order to be used on the next step           */
   int cv_qwait;                /* number of internal steps to wait before
                                   considering a change in q                   */
@@ -119,37 +191,36 @@ typedef struct CVodeMemRec {
 
   realtype cv_hin;             /* initial step size                           */
   realtype cv_h;               /* current step size                           */
-  realtype cv_hprime;          /* step size to be used on the next step       */ 
-  realtype cv_next_h;          /* step size to be used on the next step       */ 
+  realtype cv_hprime;          /* step size to be used on the next step       */
+  realtype cv_next_h;          /* step size to be used on the next step       */
   realtype cv_eta;             /* eta = hprime / h                            */
   realtype cv_hscale;          /* value of h used in zn                       */
   realtype cv_tn;              /* current internal value of t                 */
-  realtype cv_tretlast;        /* value of tret last returned by CVode        */
+  realtype cv_tretlast;        /* last value of t returned by CVode           */
 
-  realtype cv_tau[L_MAX+1];    /* array of previous q+1 successful step 
+  realtype cv_tau[L_MAX+1];    /* array of previous q+1 successful step
                                   sizes indexed from 1 to q+1                 */
-  realtype cv_tq[NUM_TESTS+1]; /* array of test quantities indexed from 
+  realtype cv_tq[NUM_TESTS+1]; /* array of test quantities indexed from
                                   1 to NUM_TESTS(=5)                          */
   realtype cv_l[L_MAX];        /* coefficients of l(x) (degree q poly)        */
 
-  realtype cv_rl1;              /* the scalar 1/l[1]                          */
-  realtype cv_gamma;            /* gamma = h * rl1                            */
-  realtype cv_gammap;           /* gamma at the last setup call               */
-  realtype cv_gamrat;           /* gamma / gammap                             */
+  realtype cv_rl1;             /* the scalar 1/l[1]                           */
+  realtype cv_gamma;           /* gamma = h * rl1                             */
+  realtype cv_gammap;          /* gamma at the last setup call                */
+  realtype cv_gamrat;          /* gamma / gammap                              */
 
-  realtype cv_crate;            /* estimated corrector convergence rate       */
-  realtype cv_acnrm;            /* | acor | wrms                              */
-  realtype cv_nlscoef;          /* coeficient in nonlinear convergence test   */
-  int  cv_mnewt;                /* Newton iteration counter                   */
+  realtype cv_crate;           /* estimated corrector convergence rate        */
+  realtype cv_delp;            /* norm of previous nonlinear solver update    */
+  realtype cv_acnrm;           /* | acor |                                    */
+  booleantype cv_acnrmcur;     /* is | acor | current?                        */
+  realtype cv_nlscoef;         /* coeficient in nonlinear convergence test    */
 
   /*------
-    Limits 
+    Limits
     ------*/
 
   int cv_qmax;          /* q <= qmax                                          */
   long int cv_mxstep;   /* maximum number of internal steps for one user call */
-  int cv_maxcor;        /* maximum number of corrector iterations for the
-                           solution of the nonlinear equation                 */
   int cv_mxhnil;        /* maximum number of warning messages issued to the
                            user that t + h == t for the next internal step    */
   int cv_maxnef;        /* maximum number of error test failures              */
@@ -160,51 +231,65 @@ typedef struct CVodeMemRec {
   realtype cv_etamax;   /* eta <= etamax                                      */
 
   /*--------
-    Counters 
+    Counters
     --------*/
 
   long int cv_nst;         /* number of internal steps taken                  */
   long int cv_nfe;         /* number of f calls                               */
   long int cv_ncfn;        /* number of corrector convergence failures        */
+  long int cv_nni;         /* number of nonlinear iterations performed        */
   long int cv_netf;        /* number of error test failures                   */
-  long int cv_nni;         /* number of Newton iterations performed           */
   long int cv_nsetups;     /* number of setup calls                           */
-  int cv_nhnil;            /* number of messages issued to the user that 
+  int cv_nhnil;            /* number of messages issued to the user that
                               t + h == t for the next iternal step            */
 
+  /*----------------
+    Step size ratios
+    ----------------*/
+
   realtype cv_etaqm1;      /* ratio of new to old h for order q-1             */
   realtype cv_etaq;        /* ratio of new to old h for order q               */
   realtype cv_etaqp1;      /* ratio of new to old h for order q+1             */
 
-  /*----------------------------
-    Space requirements for CVODE 
-    ----------------------------*/
+  /*------------------
+    Space requirements
+    ------------------*/
 
-  long int cv_lrw1;        /* no. of realtype words in 1 N_Vector             */ 
-  long int cv_liw1;        /* no. of integer words in 1 N_Vector              */ 
-  long int cv_lrw;         /* no. of realtype words in CVODE work vectors     */
-  long int cv_liw;         /* no. of integer words in CVODE work vectors      */
+  sunindextype cv_lrw1;        /* no. of realtype words in 1 N_Vector y           */
+  sunindextype cv_liw1;        /* no. of integer words in 1 N_Vector y            */
+  long int cv_lrw;             /* no. of realtype words in CVODE work vectors     */
+  long int cv_liw;             /* no. of integer words in CVODE work vectors      */
+
+  /*---------------------
+    Nonlinear Solver Data
+    ---------------------*/
+
+  SUNNonlinearSolver NLS;      /* nonlinear solver object                   */
+  booleantype ownNLS;          /* flag indicating NLS ownership             */
+  int convfail;                /* flag to indicate when a Jacobian update may
+                                  be needed */
 
   /*------------------
-    Linear Solver Data 
+    Linear Solver Data
     ------------------*/
 
   /* Linear Solver functions to be called */
 
   int (*cv_linit)(struct CVodeMemRec *cv_mem);
 
-  int (*cv_lsetup)(struct CVodeMemRec *cv_mem, int convfail, N_Vector ypred,
-		   N_Vector fpred, booleantype *jcurPtr, N_Vector vtemp1,
-		   N_Vector vtemp2, N_Vector vtemp3); 
+  int (*cv_lsetup)(struct CVodeMemRec *cv_mem, int convfail,
+                   N_Vector ypred, N_Vector fpred, booleantype *jcurPtr,
+                   N_Vector vtemp1, N_Vector vtemp2, N_Vector vtemp3);
 
   int (*cv_lsolve)(struct CVodeMemRec *cv_mem, N_Vector b, N_Vector weight,
 		   N_Vector ycur, N_Vector fcur);
 
-  void (*cv_lfree)(struct CVodeMemRec *cv_mem);
+  int (*cv_lfree)(struct CVodeMemRec *cv_mem);
 
   /* Linear Solver specific memory */
 
-  void *cv_lmem;           
+  void     *cv_lmem;  /* linear solver interface memory structure */
+  long int  cv_msbp;  /* max number of steps between lsetip calls */
 
   /*------------
     Saved Values
@@ -215,23 +300,33 @@ typedef struct CVodeMemRec {
   realtype cv_h0u;             /* actual initial stepsize                     */
   realtype cv_hu;              /* last successful h value used                */
   realtype cv_saved_tq5;       /* saved value of tq[5]                        */
-  booleantype cv_jcur;         /* is Jacobian info. for lin. solver current?  */
+  booleantype cv_jcur;         /* is Jacobian info for linear solver current? */
   realtype cv_tolsf;           /* tolerance scale factor                      */
-  int cv_qmax_alloc;           /* value of qmax used when allocating memory   */
+  int cv_qmax_alloc;           /* value of qmax used when allocating mem      */
   int cv_indx_acor;            /* index of the zn vector with saved acor      */
-  booleantype cv_setupNonNull; /* does setup do anything?                     */
+
+  /*--------------------------------------------------------------------
+    Flags turned ON by CVodeInit and read by CVodeReInit
+    --------------------------------------------------------------------*/
 
   booleantype cv_VabstolMallocDone;
-  booleantype cv_MallocDone;  
+  booleantype cv_MallocDone;
+  booleantype cv_constraintsMallocDone;
 
   /*-------------------------------------------
-    Error handler function and error ouput file 
+    Error handler function and error ouput file
     -------------------------------------------*/
 
   CVErrHandlerFn cv_ehfun;    /* error messages are handled by ehfun          */
   void *cv_eh_data;           /* data pointer passed to ehfun                 */
   FILE *cv_errfp;             /* CVODE error messages are sent to errfp       */
 
+  /*-------------------------------------------
+    User access function
+    -------------------------------------------*/
+  CVMonitorFn cv_monitorfun;     /* func called with CVODE mem and user data  */
+  long int cv_monitor_interval;  /* step interval to call cv_monitorfun       */
+
   /*-------------------------
     Stability Limit Detection
     -------------------------*/
@@ -256,13 +351,30 @@ typedef struct CVodeMemRec {
   realtype *cv_ghi;        /* saved array of g values at t = thi              */
   realtype *cv_grout;      /* array of g values at t = trout                  */
   realtype cv_toutc;       /* copy of tout (if NORMAL mode)                   */
-  realtype cv_ttol;        /* tolerance on root location                      */
+  realtype cv_ttol;        /* tolerance on root location trout                */
   int cv_taskc;            /* copy of parameter itask                         */
   int cv_irfnd;            /* flag showing whether last step had a root       */
   long int cv_nge;         /* counter for g evaluations                       */
   booleantype *cv_gactive; /* array with active/inactive event functions      */
   int cv_mxgnull;          /* number of warning messages about possible g==0  */
 
+  /*---------------
+    Projection Data
+    ---------------*/
+
+  CVodeProjMem proj_mem;      /* projection memory structure               */
+  booleantype  proj_enabled;  /* flag indicating if projection is enabled  */
+  booleantype  proj_applied;  /* flag indicating if projection was applied */
+  realtype     cv_p[L_MAX];   /* coefficients of p(x) (degree q poly)      */
+
+  /*-----------------------
+    Fused Vector Operations
+    -----------------------*/
+
+  realtype cv_cvals[L_MAX]; /* array of scalars */
+  N_Vector cv_Xvecs[L_MAX]; /* array of vectors */
+
+  booleantype cv_usefused;  /* flag indicating if CVODE specific fused kernels should be used */
 
 } *CVodeMem;
 
@@ -271,7 +383,7 @@ typedef struct CVodeMemRec {
  *     I N T E R F A C E   T O    L I N E A R   S O L V E R S
  * =================================================================
  */
-  
+
 /*
  * -----------------------------------------------------------------
  * Communication between CVODE and a CVODE Linear Solver
@@ -280,31 +392,31 @@ typedef struct CVodeMemRec {
  *
  * CV_NO_FAILURES : Either this is the first cv_setup call for this
  *                  step, or the local error test failed on the
- *                  previous attempt at this step (but the Newton
- *                  iteration converged).
+ *                  previous attempt at this step (but the nonlinear
+ *                  solver iteration converged).
  *
  * CV_FAIL_BAD_J  : This value is passed to cv_lsetup if
  *
- *                  (a) The previous Newton corrector iteration
+ *                  (a) The previous nonlinear solver corrector iteration
  *                      did not converge and the linear solver's
  *                      setup routine indicated that its Jacobian-
  *                      related data is not current
  *                                   or
- *                  (b) During the previous Newton corrector
+ *                  (b) During the previous nonlinear solver corrector
  *                      iteration, the linear solver's solve routine
  *                      failed in a recoverable manner and the
  *                      linear solver's setup routine indicated that
  *                      its Jacobian-related data is not current.
  *
  * CV_FAIL_OTHER  : During the current internal step try, the
- *                  previous Newton iteration failed to converge
+ *                  previous nonlinear solver iteration failed to converge
  *                  even though the linear solver was using current
  *                  Jacobian-related data.
  * -----------------------------------------------------------------
  */
-  
+
 /* Constants for convfail (input to cv_lsetup) */
-  
+
 #define CV_NO_FAILURES 0
 #define CV_FAIL_BAD_J  1
 #define CV_FAIL_OTHER  2
@@ -321,7 +433,7 @@ typedef struct CVodeMemRec {
  * the error handler function.
  * -----------------------------------------------------------------
  */
-  
+
 /*
  * -----------------------------------------------------------------
  * int (*cv_lsetup)(CVodeMem cv_mem, int convfail, N_Vector ypred,
@@ -351,12 +463,12 @@ typedef struct CVodeMemRec {
  * fpred - f(tn, ypred).
  *
  * jcurPtr - a pointer to a boolean to be filled in by cv_lsetup.
- *           The function should set *jcurPtr=TRUE if its Jacobian
+ *           The function should set *jcurPtr=SUNTRUE if its Jacobian
  *           data is current after the call and should set
- *           *jcurPtr=FALSE if its Jacobian data is not current.
+ *           *jcurPtr=SUNFALSE if its Jacobian data is not current.
  *           Note: If cv_lsetup calls for re-evaluation of
  *           Jacobian data (based on convfail and CVODE state
- *           data), it should return *jcurPtr=TRUE always;
+ *           data), it should return *jcurPtr=SUNTRUE always;
  *           otherwise an infinite loop can result.
  *
  * vtemp1 - temporary N_Vector provided for use by cv_lsetup.
@@ -389,38 +501,57 @@ typedef struct CVodeMemRec {
 
 /*
  * -----------------------------------------------------------------
- * void (*cv_lfree)(CVodeMem cv_mem);
+ * int (*cv_lfree)(CVodeMem cv_mem);
  * -----------------------------------------------------------------
  * cv_lfree should free up any memory allocated by the linear
  * solver. This routine is called once a problem has been
- * completed and the linear solver is no longer needed.
+ * completed and the linear solver is no longer needed.  It should
+ * return 0 upon success, nonzero on failure.
  * -----------------------------------------------------------------
  */
-  
+
 /*
  * =================================================================
- *   C V O D E    I N T E R N A L   F U N C T I O N S
+ *    I N T E R N A L   F U N C T I O N S
  * =================================================================
  */
 
 /* Prototype of internal ewtSet function */
 
-int CVEwtSet(N_Vector ycur, N_Vector weight, void *data);
+int cvEwtSet(N_Vector ycur, N_Vector weight, void *data);
 
 /* High level error handler */
 
-void CVProcessError(CVodeMem cv_mem, 
-		    int error_code, const char *module, const char *fname, 
-		    const char *msgfmt, ...);
+void cvProcessError(CVodeMem cv_mem, int error_code, const char *module,
+                    const char *fname, const char *msgfmt, ...);
 
-/* Prototype of internal errHandler function */
+/* Prototype of internal ErrHandler function */
 
-void CVErrHandler(int error_code, const char *module, const char *function, 
-		  char *msg, void *data);
+void cvErrHandler(int error_code, const char *module, const char *function,
+                  char *msg, void *data);
+
+/* Nonlinear solver initialization */
+
+int cvNlsInit(CVodeMem cv_mem);
+
+/* Projection functions */
+
+int cvDoProjection(CVodeMem cv_mem, int *nflagPtr, realtype saved_t,
+                   int *npfPtr);
+int cvProjInit(CVodeProjMem proj_mem);
+int cvProjFree(CVodeProjMem *proj_mem);
+
+/* Restore tn and undo prediction to reattempt a step */
+
+void cvRestore(CVodeMem cv_mem, realtype saved_t);
+
+/* Reset h and rescale history array to prepare for a step */
+
+void cvRescale(CVodeMem cv_mem);
 
 /*
  * =================================================================
- *   C V O D E    E R R O R    M E S S A G E S
+ *    E R R O R    M E S S A G E S
  * =================================================================
  */
 
@@ -456,7 +587,6 @@ void CVErrHandler(int error_code, const char *module, const char *function,
 #define MSGCV_CVMEM_FAIL "Allocation of cvode_mem failed."
 #define MSGCV_MEM_FAIL "A memory request failed."
 #define MSGCV_BAD_LMM  "Illegal value for lmm. The legal values are CV_ADAMS and CV_BDF."
-#define MSGCV_BAD_ITER  "Illegal value for iter. The legal values are CV_FUNCTIONAL and CV_NEWTON."
 #define MSGCV_NO_MALLOC "Attempt to call before CVodeInit."
 #define MSGCV_NEG_MAXORD "maxord <= 0 illegal."
 #define MSGCV_BAD_MAXORD  "Illegal attempt to increase maximum method order."
@@ -468,17 +598,20 @@ void CVErrHandler(int error_code, const char *module, const char *function,
 #define MSGCV_BAD_ABSTOL "abstol has negative component(s) (illegal)."
 #define MSGCV_NULL_ABSTOL "abstol = NULL illegal."
 #define MSGCV_NULL_Y0 "y0 = NULL illegal."
+#define MSGCV_Y0_FAIL_CONSTR "y0 fails to satisfy constraints."
 #define MSGCV_NULL_F "f = NULL illegal."
 #define MSGCV_NULL_G "g = NULL illegal."
 #define MSGCV_BAD_NVECTOR "A required vector operation is not implemented."
+#define MSGCV_BAD_CONSTR "Illegal values in constraints vector."
 #define MSGCV_BAD_K "Illegal value for k."
 #define MSGCV_NULL_DKY "dky = NULL illegal."
 #define MSGCV_BAD_T "Illegal value for t." MSG_TIME_INT
 #define MSGCV_NO_ROOT "Rootfinding was not initialized."
+#define MSGCV_NLS_INIT_FAIL "The nonlinear solver's init routine failed."
 
 /* CVode Error Messages */
 
-#define MSGCV_NO_TOLS "No integration tolerances have been specified."
+#define MSGCV_NO_TOL "No integration tolerances have been specified."
 #define MSGCV_LSOLVE_NULL "The linear solver's solve routine is NULL."
 #define MSGCV_YOUT_NULL "yout = NULL illegal."
 #define MSGCV_TRET_NULL "tret = NULL illegal."
@@ -499,6 +632,7 @@ void CVErrHandler(int error_code, const char *module, const char *function,
 #define MSGCV_CONV_FAILS "At " MSG_TIME_H ", the corrector convergence test failed repeatedly or with |h| = hmin."
 #define MSGCV_SETUP_FAILED "At " MSG_TIME ", the setup routine failed in an unrecoverable manner."
 #define MSGCV_SOLVE_FAILED "At " MSG_TIME ", the solve routine failed in an unrecoverable manner."
+#define MSGCV_FAILED_CONSTR "At " MSG_TIME ", unable to satisfy inequality constraints."
 #define MSGCV_RHSFUNC_FAILED "At " MSG_TIME ", the right-hand side routine failed in an unrecoverable manner."
 #define MSGCV_RHSFUNC_UNREC "At " MSG_TIME ", the right-hand side failed in a recoverable manner, but no recovery is possible."
 #define MSGCV_RHSFUNC_REPTD "At " MSG_TIME " repeated recoverable right-hand side function errors."
@@ -507,6 +641,18 @@ void CVErrHandler(int error_code, const char *module, const char *function,
 #define MSGCV_CLOSE_ROOTS "Root found at and very near " MSG_TIME "."
 #define MSGCV_BAD_TSTOP "The value " MSG_TIME_TSTOP " is behind current " MSG_TIME " in the direction of integration."
 #define MSGCV_INACTIVE_ROOTS "At the end of the first step, there are still some root functions identically 0. This warning will not be issued again."
+#define MSGCV_NLS_SETUP_FAILED "At " MSG_TIME ", the nonlinear solver setup failed unrecoverably."
+#define MSGCV_NLS_INPUT_NULL "At " MSG_TIME ", the nonlinear solver was passed a NULL input."
+#define MSGCV_NLS_FAIL "At " MSG_TIME ", the nonlinear solver failed in an unrecoverable manner."
+
+/* CVode Projection Error Messages */
+
+#define MSG_CV_MEM_NULL  "cvode_mem = NULL illegal."
+#define MSG_CV_MEM_FAIL  "A memory request failed."
+
+#define MSG_CV_PROJ_MEM_NULL       "proj_mem = NULL illegal."
+#define MSG_CV_PROJFUNC_FAIL       "At " MSG_TIME " the projection function failed with an unrecoverable error."
+#define MSG_CV_REPTD_PROJFUNC_ERR  "At " MSG_TIME " the projection function had repeated recoverable errors."
 
 #ifdef __cplusplus
 }
